VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CDDAWriter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True

' CDDA Writer Active X DLL
' Copyright (C) 2005  [rm_code]

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


Option Explicit

Private Declare Function GetTempPath Lib "kernel32" _
Alias "GetTempPathA" ( _
    ByVal nBufferLength As Long, _
    ByVal lpBuffer As String _
) As Long

'write progress
Public Event WriteProgress(ByVal percent As Integer, ByVal track As Integer)
Public Event ConvProgress(ByVal percent As Integer, ByVal track As Integer)

Public Enum WriteFeatures
    WRT_CDR = 1                             ' can write to CD-R?
    WRT_CDRW = 2                            ' can write to CD-RW?
    WRT_TESTMODE = 4                        ' can turn off laser?
End Enum

Public Enum DiscTypes
    CD_CDR                                  ' CD Type  CD-R
    CD_CDRW                                 '          CD-RW
    CD_CDROM                                '          CD-ROM
    CD_DVD                                  '          DVD
    CD_NOTPRESENT                           ' no disc present
End Enum

Public Enum DiscStatus
    DS_EMPTY                                ' empty
    DS_INCOMPLETE                           ' uncomplete
    DS_COMPLETE                             ' complete
    DS_UNKNWN                               ' unknown
End Enum

Private WithEvents WavConvert As clsConvert ' Wave/MP3 Converter
Attribute WavConvert.VB_VarHelpID = -1
Private WithEvents WrtAudio   As clsAudioCD ' Audio CD writer
Attribute WrtAudio.VB_VarHelpID = -1

Private strDrvID              As String     ' current drive ID

Private blnTest               As Boolean    ' Test Mode
Private blnMultiSession       As Boolean    ' allow multi session
Private blnEjectAfterWrite    As Boolean    ' eject disc after writing
Private lngWriteSpeed         As Long       ' write speed
Private intPregap             As Integer    ' Pre-Gap size

'burn files as audio tracks
Public Function BurnDisc(ByRef files() As String, _
                         Optional ByRef cancel As Boolean) As eBurnReturn

    BurnDisc = WrtAudio.BurnAudio(strDrvID, files, lngWriteSpeed, _
                                  blnTest, intPregap, blnMultiSession, _
                                  blnEjectAfterWrite, cancel)

End Function

'eject disc after writing
Public Property Get DiscEjectAfterWrite() As Boolean

    DiscEjectAfterWrite = blnEjectAfterWrite

End Property

'eject disc after writing
Public Property Let DiscEjectAfterWrite(ByVal val As Boolean)

    blnEjectAfterWrite = val

End Property

'allow multi session
Public Property Get DiscAllowMultiSession() As Boolean

    DiscAllowMultiSession = blnMultiSession

End Property

'allow multi session
Public Property Let DiscAllowMultiSession(ByVal val As Boolean)

    blnMultiSession = val

End Property

'Pre-Gap size
Public Property Get PreGapLength() As Integer

    PreGapLength = intPregap \ 75

End Property

'Pre-Gap size
Public Property Let PreGapLength(ByVal val As Integer)

    intPregap = val * 75

End Property

'writespeed
Public Property Get WriteSpeed() As eWriteSpeeds

    WriteSpeed = lngWriteSpeed

End Property

'writespeed
Public Property Let WriteSpeed(ByVal val As eWriteSpeeds)

    lngWriteSpeed = val

End Property

'Test Mode
Public Property Get TestMode() As Boolean

    TestMode = blnTest

End Property

'Test Mode
Public Property Let TestMode(ByVal val As Boolean)

    blnTest = val

End Property

'temporary wave temp
Public Property Get WaveTemp() As String

    WaveTemp = WrtAudio.WaveTemp

End Property

'temporary wave temp
Public Property Let WaveTemp(ByVal val As String)

    WrtAudio.WaveTemp = val

End Property

'Status of the disc
Public Property Get CDStatus() As DiscStatus

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    CDStatus = CDRomGetCDInfo(strDrvID).DiscStatus

End Property

'capacity of the disc
Public Property Get CDCapacity() As Double

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    CDCapacity = CDbl(CDRomGetCDInfo(strDrvID).LeadOut.LBA) * 2352

End Property

'Type of the inserted disc
Public Property Get CDType() As DiscTypes

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    With CDRomGetCDInfo(strDrvID)
        Select Case .CDType

            'CD-R
            Case ROMTYPE_CDR

                CDType = CD_CDR

            'CD-RW
            Case ROMTYPE_CDRW

                CDType = CD_CDRW

            'DVD
            Case ROMTYPE_DVD_P_R, ROMTYPE_DVD_R, _
                 ROMTYPE_DVD_P_RW, ROMTYPE_DVD_RAM, _
                 ROMTYPE_DVD_ROM, ROMTYPE_DVD_RW

                CDType = CD_DVD

            'CD-ROM
            Case ROMTYPE_CDROM

                CDType = CD_CDROM

            'no media or CD-ROM
            Case ROMTYPE_CDROM_R_RW

                If CDRomIsDiscPresent(strDrvID) < 1 Then
                    CDType = CD_NOTPRESENT
                Else
                    CDType = CD_CDROM
                End If

        End Select
    End With

End Property

'buffer size
Public Property Get DriveBufferSize() As Long

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    DriveBufferSize = CDRomGetLWInfo(strDrvID).BufferSize

End Property

'write features
Public Property Get DriveWriteFeatures() As WriteFeatures

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    With CDRomGetLWInfo(strDrvID)

        If .WriteFeatures.CDR Then _
            DriveWriteFeatures = DriveWriteFeatures Or WRT_CDR

        If .WriteFeatures.CDRW Then _
            DriveWriteFeatures = DriveWriteFeatures Or WRT_CDRW

        If .WriteFeatures.TestMode Then _
            DriveWriteFeatures = DriveWriteFeatures Or WRT_TESTMODE

    End With

End Property

'current write speed of the drive
Public Property Get DriveCurrWriteSpeed() As Long

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    DriveCurrWriteSpeed = CDRomGetSpeed(strDrvID).CurrWSpeed

End Property

'max. write speed of the drive
Public Property Get DriveMaxWriteSpeed() As Long

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    DriveMaxWriteSpeed = CDRomGetSpeed(strDrvID).MaxWSpeed

End Property

'selected drive is ready?
Public Property Get DriveReady() As Boolean

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    DriveReady = CDRomTestUnitReady(strDrvID)       ' Test Unit Ready

End Property

'disc present?
Public Property Get DiscPresent() As Boolean

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    DiscPresent = CDRomIsDiscPresent(strDrvID)      ' CD present?

End Property

'eject media
Public Function DiscEject() As Boolean

    If strDrvID = vbNullString Then Exit Function   ' drive selected?

    DiscEject = CDRomUnloadTray(strDrvID)           ' eject

End Function

'load media
Public Function DiscLoad() As Boolean

    If strDrvID = vbNullString Then Exit Function   ' drive selected?

    DiscLoad = CDRomLoadTray(strDrvID)              ' load

End Function

'tray open?
Public Property Get DriveOpen() As Boolean

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    DriveOpen = CDRomIsTrayOpen(strDrvID)           ' Tray Check

End Property

'drive char of the selected drive
Public Property Get DriveChar() As String

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    DriveChar = Left$(cd.DrvChrFromDrvID(strDrvID), 1)

End Property

'name of the currently selected drive
Public Property Get DriveName() As String

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    DriveName = cd.CDROMGetName(strDrvID, Vendor Or Product Or revision)

End Property

'host adapter of the currently selected drive
Public Property Get DriveHA() As Integer

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    DriveHA = Asc(Left$(cd.HAIDFromDrvChr(cd.DrvChrFromDrvID(strDrvID)), 1))

End Property

'target of the currently selected drive
Public Property Get DriveTarget() As Integer

    If strDrvID = vbNullString Then Exit Property   ' drive selected?

    DriveTarget = Asc(Right$(cd.HAIDFromDrvChr(cd.DrvChrFromDrvID(strDrvID)), 1))

End Property

'selects a CD/DVD-ROM drive
Public Function SelectCDROM(ByVal strDrive As String) As Boolean

    If Not IsCDROM(strDrive) Then Exit Function ' CD/DVD-ROM LW?

    strDrvID = cd.DrvIDFromDrvChr(strDrive)

    SelectCDROM = True                          ' OK

End Function

'determines if the drive type is CD/DVD-ROM
Public Function IsCDROM(ByVal strDrive As String) As Boolean

    strDrive = Left$(strDrive, 1) & ":"

    IsCDROM = cd.IsCDROM(strDrive)

End Function

'selected interface (ASPI or SPTI)
Public Property Get SelectedInterface() As String

    SelectedInterface = Choose(cd.CurrentInterface, "ASPI", "SPTI")

End Property

'constructor
Private Sub Class_Initialize()

    Set WavConvert = New clsConvert
    Set WrtAudio = New clsAudioCD

    WaveTemp = GetTempDir                   ' temp

    WriteSpeed = [8x]                       ' write speed

    cd.ChooseInterface                      ' ASPI or SPTI

    ListCodecs                              ' search for ACM MP3 Codecs

End Sub

Private Sub WrtAudio_ConvProgress(ByVal percent As Integer, _
                                  ByVal track As Integer)

    'audio conversion progress
    RaiseEvent ConvProgress(percent, track)

End Sub

Private Sub WrtAudio_WriteProgress(ByVal percent As Integer, _
                                   ByVal track As Integer)

    'write progress
    RaiseEvent WriteProgress(percent, track)

End Sub

'temp dir
Private Function GetTempDir() As String

    GetTempDir = Space(255)
    GetTempDir = Left$(GetTempDir, GetTempPath(255, GetTempDir))
    If Not Right$(GetTempDir, 1) = "\" Then GetTempDir = GetTempDir & "\"

End Function

'get the duration of MP3s/WAVs
Public Function AudioLength(ByVal strFileName As String) As Long

    Select Case LCase$(Right$(strFileName, 3))

        Case "wav": AudioLength = GetWAVLength(strFileName)
        Case "mp3": AudioLength = GetMP3Length(strFileName)

    End Select

End Function
