VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsAudioCD"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True

' CDDA Writer Active X DLL
' Copyright (C) 2005  [rm_code]

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.



'//////////////////////////
' Audio CD Writer class
'
' [rm_code] 2005
'\\\\\\\\\\\\\\\\\\\\\\\\\\


Option Explicit             ' explicit parsing


Public Enum eBurnReturn
    ERR_DISC_NOT_SUPPORTED  ' CD Type not supported
    ERR_TOO_MANY_TRACKS     ' too many tracks (max. 99)
    ERR_DISC_NOT_EMPTY      ' CD not empty
    ERR_DISC_MISSING        ' no disc present
    ERR_WRITE_CANCEL        ' writing canceled
    ERR_CLOSE_TRACK         ' couldn't close track
    ERR_WRITE_ERROR         ' write error (Buffer-Underrun?)
    ERR_NO_CDWRITER         ' drive is not a CD writer
    ERR_WRITESPEED          ' couldn't set write speed
    ERR_CLOSE_DISC          ' couldn't close disc
    ERR_NEXT_ADDR           ' couldn't get next writable adr
    ERR_CAPACITY            ' data bigger than disc's capacity
    ERR_TESTMODE            ' drive doesn't support test mode
    ERR_DRIVEID             ' drive ID missing
    ERR_WPP                 ' couldn't send Write Parameters Page
    ERR_OK                  ' finished
End Enum

'supported write speeds
Public Enum eWriteSpeeds
     [2x] = 2 * 176
     [4x] = 4 * 176
     [8x] = 8 * 176
    [12x] = 12 * 176
    [16x] = 16 * 176
End Enum

'     blocks per read
Const BPL                   As Long = 8

'Wave/MP3 Converter
Private WithEvents wavconv  As clsConvert
Attribute wavconv.VB_VarHelpID = -1

'temp dir
Private strTemp             As String

'current track
Private CurrTrack           As Integer

'progress events
Public Event WriteProgress(ByVal percent As Integer, ByVal track As Integer)
Public Event ConvProgress(ByVal percent As Integer, ByVal track As Integer)

Public Property Get WaveTemp() As String

    WaveTemp = strTemp

End Property

Public Property Let WaveTemp(ByVal val As String)

    If Not Right$(val, 1) = "\" Then val = val & "\"

    If Not FolderExists(val) Then Exit Property

    strTemp = val

End Property

Public Function BurnAudio(ByVal strDriveID As String, _
                          ByRef files() As String, _
                          Optional ByVal WriteSpeed As eWriteSpeeds = [8x], _
                          Optional ByVal TestMode As Boolean, _
                          Optional ByVal PreGap As Integer = 150, _
                          Optional ByVal MultiSession As Boolean, _
                          Optional ByVal EjectDisc As Boolean = True, _
                          Optional ByRef cancel As Boolean) As eBurnReturn

    '   Dateihandle
    Dim FF          As Integer: FF = FreeFile
    '   counter                     counter
    Dim i           As Long:    Dim j       As Long

    '   next writable address
    Dim startLBA    As Long
    '   written sectors
    Dim written  As Long

    '   data size            progress
    Dim lngCDSize   As Long, intPercent As Integer

    '   read buffer
    Dim buffer(CLng(2352) * BPL - 1) As Byte

    '   Read Track Information structure
    Dim rti As t_RTI


    'data size in bytes
    For i = LBound(files) To UBound(files)
        lngCDSize = lngCDSize + AudioLength(files(i)) * 176400
    Next


    'Drive ID submitted?
    If strDriveID = vbNullString Then _
        BurnAudio = ERR_DRIVEID: Exit Function

    'Max. 99 Tracks
    If UBound(files) > 98 Then _
        BurnAudio = ERR_TOO_MANY_TRACKS: Exit Function

    'CD present?
    If CDRomIsDiscPresent(strDriveID) < 1 Then _
        BurnAudio = ERR_DISC_MISSING: Exit Function


    With CDRomGetLWInfo(strDriveID).WriteFeatures

        'drive can write to CD-R/RW?
        If Not (.CDR Or .CDRW) Then _
            BurnAudio = ERR_NO_CDWRITER: Exit Function

        'drive supports test mode?
        If TestMode Then _
            If Not .TestMode Then _
                BurnAudio = ERR_TESTMODE: Exit Function

    End With


    With CDRomGetCDInfo(strDriveID)

        'CD type supported?
        If Not (.CDType = ROMTYPE_CDR Or _
                .CDType = ROMTYPE_CDRW Or _
                .CDType = ROMTYPE_CDROM_R_RW) Then

            BurnAudio = ERR_DISC_NOT_SUPPORTED
            Exit Function

        End If

        'CD empty?
        If .DiscStatus <> STAT_EMPTY Then _
            BurnAudio = ERR_DISC_NOT_EMPTY: Exit Function

        'data fits to CD?
        If .Capacity <= lngCDSize \ 2352 Then _
            BurnAudio = ERR_CAPACITY: Exit Function

    End With


    'set the new write speed
    If Not CDRomSetCDSpeed(strDriveID, &HFFFF&, WriteSpeed, False) Then _
        BurnAudio = ERR_WRITESPEED: Exit Function


    'Laser calibration
    'mustn't work..., but CDR-DAO does it, too ;)
    If Not CDRomSendOPCInformation(strDriveID, True, 0, 0) Then _
        Debug.Print "OPC failed"



    'start the main loop
    For i = 0 To UBound(files)

        CurrTrack = i + 1

        'convert MP3 or Wave to 44.1 kHz 16-Bit stereo
        wavconv.Source = files(i)
        wavconv.Destination = strTemp & "tmp.wav"
        wavconv.Convert

        'send the new Write Parameters Page.
        'The Write Parameters Page tells the drive how it should act.
        'Track-At-Once is the easiest mode.
        'You only send the userdata of a sector,
        'the CD Writer does the rest (error correction, sub-channels).

        'We set the following mode:
        'Track-At-Once, std. 2 seconds (150 frames) Pre-Gap,
        'Stereo Audio w/o Pre-Emphasis, 2352 Bytes/Sector,
        'possibly Test-Mode, no BURN-Proof, possibly
        'allow further sessions
        If Not CDRomWriteParams(strDriveID, TestMode, False, PreGap, _
                                1, 0, 0, MultiSession) Then

            BurnAudio = ERR_WPP
            Exit Function

        End If

        'read information about the current track.
        'we want the next writable address.
        If Not CDRomReadTrackInformation(strDriveID, 1, _
                                         i + 1, _
                                         VarPtr(rti), _
                                         Len(rti) - 1) Then

            BurnAudio = ERR_NEXT_ADDR
            Exit Function

        End If

        'next writable address as a LBA value
        startLBA = cd.LShift(rti.Track_Next_Writable(0), 24) Or _
                   cd.LShift(rti.Track_Next_Writable(1), 16) Or _
                   cd.LShift(rti.Track_Next_Writable(2), 8) Or _
                   rti.Track_Next_Writable(3)

        'reset the track position
        written = 0

        'open the file
        Open wavconv.Destination For Binary As #FF

            'jumpf over the Wave Header
            Seek FF, 45

            'read the file
            For j = 0 To LOF(FF) Step BPL * 2352

                'cancel burning?
                If cancel Then
                    cancel = False

                    'yes, close the disc
                    BurnAudio = ERR_WRITE_CANCEL
                    If Not TestMode Then CDRomCloseCD strDriveID, 1, i + 1
                    CDRomSyncCache strDriveID
                    If EjectDisc Then CDRomUnloadTray strDriveID
                    Close #FF

                    Exit Function
                End If

                'fill the read buffer
                Get #FF, , buffer

                'write it to the drive's buffer
                If Not Burn(strDriveID, _
                            startLBA + written, BPL, _
                            VarPtr(buffer(0)), _
                            UBound(buffer)) Then

                    BurnAudio = ERR_WRITE_ERROR
                    If Not TestMode Then CDRomCloseCD strDriveID, 1, i + 1
                    If EjectDisc Then CDRomUnloadTray strDriveID
                    Close #FF
                    Exit Function

                End If

                'jump to the next writable address in the current track
                written = written + BPL

                'calculate the progress in percent
                '
                'because of Track Pre-Gaps check for overflows.
                intPercent = (startLBA + written) * 2352 / lngCDSize * 100
                If intPercent > 100 Then intPercent = 100

                RaiseEvent WriteProgress(intPercent, CurrTrack)

            Next

        Close #FF

        'write the rest from the buffer to the CD
        If Not CDRomSyncCache(strDriveID) Then _
            Debug.Print "SYNCHRONIZE CACHE failed"

        'close the track
        If Not TestMode Then

            If Not CDRomCloseCD(strDriveID, 1, i + 1) Then
                BurnAudio = ERR_CLOSE_TRACK

                'eject the media
                If EjectDisc Then _
                    If Not CDRomUnloadTray(strDriveID) Then _
                        Debug.Print "couldn't eject media."

                Exit Function
            End If

        End If

    Next

    'write a session lead-out.
    'If we have set the WPP to "no multi session",
    'the disc should get finalized.
    If Not TestMode Then

        If Not CDRomCloseCD(strDriveID, 2, 0) Then
            BurnAudio = ERR_CLOSE_DISC
    
            'eject the media
            If EjectDisc Then _
                If Not CDRomUnloadTray(strDriveID) Then _
                    Debug.Print "couldn't eject media."
    
            Exit Function
        End If

    End If

    'eject the media
    If EjectDisc Then _
        If Not CDRomUnloadTray(strDriveID) Then _
            Debug.Print "couldn't eject media."

    'finished
    BurnAudio = ERR_OK

End Function

'forever try to write to the drive buffer,
'to avoid buffer overflows.
'also look for buffer underruns.
Private Function Burn(ByVal DriveID As String, _
                      ByVal LBA As Long, ByVal sectors As Long, _
                      ByVal PtrBuf As Long, ByVal BufLen As Long _
                     ) As Boolean

    '   buffer structure
    Dim bufcap      As t_BufferCapacity
    '   buffer length        empty part of the buffer
    Dim BufferLen   As Long, BufferBlank    As Long


    'as long as writing doesn't work
    Do While Not CDRomWrite10(DriveID, LBA, sectors, PtrBuf, BufLen)

        'read buffer stats
        If Not CDRomBufferCapacity(DriveID, VarPtr(bufcap), _
                                   Len(bufcap) - 1) Then

            Exit Function

        Else

            'buffer len
            BufferLen = cd.LShift(bufcap.BufferLen(0), 24) Or _
                        cd.LShift(bufcap.BufferLen(1), 16) Or _
                        cd.LShift(bufcap.BufferLen(2), 8) Or _
                                  bufcap.BufferLen(3)

            'empty part of the buffer
            BufferBlank = cd.LShift(bufcap.BufferBlank(0), 24) Or _
                          cd.LShift(bufcap.BufferBlank(1), 16) Or _
                          cd.LShift(bufcap.BufferBlank(2), 8) Or _
                                    bufcap.BufferBlank(3)

            'check for invalid data or buffer underruns
            If BufferBlank = BufferLen Or _
               BufferLen = 0 Or _
               BufferBlank > BufferLen Then

                'oh oh
                Exit Function

            End If

        End If

    Loop

    'success
    Burn = True

End Function

Private Sub Class_Initialize()

    Set wavconv = New clsConvert

End Sub

Private Sub wavconv_progress(ByVal percent As Integer)

    RaiseEvent ConvProgress(percent, CurrTrack)

End Sub

Private Function FolderExists(Folder As String) As Boolean

    On Error Resume Next
    FolderExists = Dir(Folder, vbDirectory) <> ""

End Function

Private Function AudioLength(ByVal strFileName As String) As Long

    Select Case LCase$(Right$(strFileName, 3))

        Case "wav": AudioLength = GetWAVLength(strFileName)
        Case "mp3": AudioLength = GetMP3Length(strFileName)

    End Select

End Function
